using System;
using System.Data;
using System.Text;

namespace ReflectionIT.Data.Sql
{
	public abstract class CommandBuilderBase {

		public virtual IDbCommand CreateCommand(TokenList clauses) {
			IDbCommand cmd = CreateCommand();

			StringBuilder sb = new StringBuilder();

			foreach (IToken token in clauses) {
				AppendToken(cmd, sb, token);
			}

			cmd.CommandText = sb.ToString();

			return cmd;
		}

		protected virtual void AppendToken(IDbCommand command, StringBuilder sql, IToken token) {
			TokenList wl = token.GetList();
			if (wl == null) {
				if (token is Constant) {
					this.Append(command, sql, (Constant)token);
				} else {
					if (token is Keyword) {
						this.Append(command, sql, (Keyword)token);
					} else {
						if (token is Identifier) {
							this.Append(command, sql, (Identifier)token);
						} else {
							this.Append(command, sql, (Literal)token);
						}
					}
				}
			} else {
				foreach (IToken word2 in wl) {
					this.AppendToken(command, sql, word2);
				}
			}
		}

		protected abstract IDbCommand CreateCommand();

		protected abstract void Append(IDbCommand command, StringBuilder sql, Constant constant);

		protected virtual void Append(IDbCommand command, StringBuilder sql, Identifier column) {
			sql.Append(column.ToString());
		}

		protected virtual void Append(IDbCommand command, StringBuilder sql, Keyword keyword) {
			sql.Append(keyword.ToString());
		}

		protected virtual void Append(IDbCommand command, StringBuilder sql, Literal literal) {
			sql.Append(literal.ToString());
		}
	}
}
